@@ -30,6 +30,7 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <errno.h>
+#include <locale.h>
 #include <glib/gi18n.h>
 
 #include <nm-setting-vpn.h>
@@ -47,6 +48,7 @@ G_DEFINE_TYPE (NMVPNCPlugin, nm_vpnc_plugin, NM_TYPE_VPN_PLUGIN)
 
 typedef struct {
 	GPid pid;
+	char *pid_file;
 } NMVPNCPluginPrivate;
 
 #define NM_VPNC_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_VPNC_PLUGIN, NMVPNCPluginPrivate))
@@ -60,6 +62,7 @@ static const char *vpnc_binary_paths[] =
 };
 
 #define NM_VPNC_HELPER_PATH		LIBEXECDIR"/nm-vpnc-service-vpnc-helper"
+#define NM_VPNC_PID_PATH		LOCALSTATEDIR"/run/NetworkManager"
 #define NM_VPNC_UDP_ENCAPSULATION_PORT	0 /* random port */
 #define NM_VPNC_LOCAL_PORT_ISAKMP	0 /* random port */
 
@@ -240,6 +243,18 @@ nm_vpnc_secrets_validate (NMSettingVPN *s_vpn, GError **error)
 	return *error ? FALSE : TRUE;
 }
 
+static void
+remove_pidfile (NMVPNCPlugin *plugin)
+{
+	NMVPNCPluginPrivate *priv = NM_VPNC_PLUGIN_GET_PRIVATE (plugin);
+
+	if (priv->pid_file) {
+		unlink (priv->pid_file);
+		g_free (priv->pid_file);
+		priv->pid_file = NULL;
+	}
+}
+
 static void
 vpnc_watch_cb (GPid pid, gint status, gpointer user_data)
 {
@@ -263,6 +278,8 @@ vpnc_watch_cb (GPid pid, gint status, gpointer user_data)
 	waitpid (priv->pid, NULL, WNOHANG);
 	priv->pid = 0;
 
+	remove_pidfile (plugin);
+
 	/* Must be after data->state is set since signals use data->state */
 	switch (error) {
 	case 2:
@@ -283,6 +300,7 @@ vpnc_watch_cb (GPid pid, gint status, gpointer user_data)
 static gint
 nm_vpnc_start_vpnc_binary (NMVPNCPlugin *plugin, GError **error)
 {
+	NMVPNCPluginPrivate *priv = NM_VPNC_PLUGIN_GET_PRIVATE (plugin);
 	GPid	pid;
 	const char **vpnc_binary = NULL;
 	GPtrArray *vpnc_argv;
@@ -310,6 +328,8 @@ nm_vpnc_start_vpnc_binary (NMVPNCPlugin *plugin, GError **error)
 	g_ptr_array_add (vpnc_argv, (gpointer) (*vpnc_binary));
 	g_ptr_array_add (vpnc_argv, (gpointer) "--non-inter");
 	g_ptr_array_add (vpnc_argv, (gpointer) "--no-detach");
+	g_ptr_array_add (vpnc_argv, (gpointer) "--pid-file");
+	g_ptr_array_add (vpnc_argv, (gpointer) priv->pid_file);
 	g_ptr_array_add (vpnc_argv, (gpointer) "-");
 	g_ptr_array_add (vpnc_argv, NULL);
 
@@ -531,6 +551,7 @@ real_connect (NMVPNPlugin   *plugin,
               NMConnection  *connection,
               GError       **error)
 {
+	NMVPNCPluginPrivate *priv = NM_VPNC_PLUGIN_GET_PRIVATE (plugin);
 	NMSettingVPN *s_vpn;
 	gint vpnc_fd = -1;
 	gboolean success = FALSE;
@@ -543,6 +564,8 @@ real_connect (NMVPNPlugin   *plugin,
 	if (!nm_vpnc_secrets_validate (s_vpn, error))
 		goto out;
 
+	priv->pid_file = g_strdup_printf (NM_VPNC_PID_PATH "/nm-vpnc-%s.pid", nm_connection_get_uuid (connection));
+
 	vpnc_fd = nm_vpnc_start_vpnc_binary (NM_VPNC_PLUGIN (plugin), error);
 	if (vpnc_fd < 0)
 		goto out;
@@ -724,17 +747,26 @@ main (int argc, char *argv[])
 		{NULL}
 	};
 
+#if !GLIB_CHECK_VERSION (2, 35, 0)
 	g_type_init ();
+#endif
+
+	/* locale will be set according to environment LC_* variables */
+	setlocale (LC_ALL, "");
+
+	bindtextdomain (GETTEXT_PACKAGE, NM_VPNC_LOCALEDIR);
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
 
 	/* Parse options */
-	opt_ctx = g_option_context_new ("");
-	g_option_context_set_translation_domain (opt_ctx, "UTF-8");
+	opt_ctx = g_option_context_new (NULL);
+	g_option_context_set_translation_domain (opt_ctx, GETTEXT_PACKAGE);
 	g_option_context_set_ignore_unknown_options (opt_ctx, FALSE);
 	g_option_context_set_help_enabled (opt_ctx, TRUE);
 	g_option_context_add_main_entries (opt_ctx, options, NULL);
 
 	g_option_context_set_summary (opt_ctx,
-		_("nm-vpnc-service provides integrated Cisco Legacy IPSec VPN capability to NetworkManager."));
+		_("nm-vpnc-service provides integrated Cisco Legacy IPsec VPN capability to NetworkManager."));
 
 	g_option_context_parse (opt_ctx, &argc, &argv, NULL);
 	g_option_context_free (opt_ctx);
@@ -752,6 +784,9 @@ main (int argc, char *argv[])
 	if (!plugin)
 		exit (EXIT_FAILURE);
 
+	if (debug)
+		g_message ("nm-vpnc-service (version " DIST_VERSION ") started.");
+
 	loop = g_main_loop_new (NULL, FALSE);
 
 	if (!persist)
@@ -760,6 +795,8 @@ main (int argc, char *argv[])
 	setup_signals ();
 	g_main_loop_run (loop);
 
+	remove_pidfile (plugin);
+
 	g_main_loop_unref (loop);
 	g_object_unref (plugin);
 
